@@ -4,22 +4,28 @@ Features
 ------
 
 Extra features Elements.
+
 """
-from jinja2 import Template
+
 import json
 
-from .utilities import (_parse_size,
-                        _locations_mirror, _locations_tolist, image_to_url,
-                        text_type, binary_type,
-                        none_min, none_max, iter_points,
-                        )
+from jinja2 import Template
+from six import text_type, binary_type
 
-from .element import Element, Figure, JavascriptLink, CssLink, MacroElement
-from .map import Layer, Icon, Marker, Popup
+from branca.utilities import (
+    _parse_size, _locations_mirror, _locations_tolist, image_to_url,
+    none_min, none_max, iter_points
+)
+from branca.element import (Element, Figure, JavascriptLink, CssLink,
+                            MacroElement)
+from branca.colormap import LinearColormap
+
+from .map import Layer, Icon, Marker, Popup, FeatureGroup
 
 
 class WmsTileLayer(Layer):
-    """Creates a Web Map Service (WMS) layer.
+    """
+    Creates a Web Map Service (WMS) layer.
 
     Parameters
     ----------
@@ -62,8 +68,6 @@ class WmsTileLayer(Layer):
         self.format = format if format else 'image/jpeg'
         self.transparent = transparent
         self.version = version
-        # FIXME: Should be map CRS!
-        # self.crs = crs if crs else 'null
         self._template = Template(u"""
         {% macro script(this, kwargs) %}
             var {{this.get_name()}} = L.tileLayer.wms(
@@ -83,7 +87,8 @@ class WmsTileLayer(Layer):
 
 
 class RegularPolygonMarker(Marker):
-    """Custom markers using the Leaflet Data Vis Framework.
+    """
+    Custom markers using the Leaflet Data Vis Framework.
 
     Parameters
     ----------
@@ -115,6 +120,7 @@ class RegularPolygonMarker(Marker):
     Polygon marker names and HTML in obj.template_vars
 
     For more information, see https://humangeo.github.io/leaflet-dvf/
+
     """
     def __init__(self, location, color='black', opacity=1, weight=2,
                  fill_color='blue', fill_opacity=1,
@@ -158,13 +164,14 @@ class RegularPolygonMarker(Marker):
         assert isinstance(figure, Figure), ("You cannot render this Element "
                                             "if it's not in a Figure.")
 
-        figure.header.add_children(
-            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet-dvf/0.2/leaflet-dvf.markers.min.js"),  # noqa
+        figure.header.add_child(
+            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet-dvf/0.3.0/leaflet-dvf.markers.min.js"),  # noqa
             name='dvf_js')
 
 
 class Vega(Element):
-    """Cretes a Vega chart element.
+    """
+    Creates a Vega chart element.
 
     Parameters
     ----------
@@ -189,6 +196,7 @@ class Vega(Element):
     position: str, default 'relative'
         The `position` argument that the CSS shall contain.
         Ex: 'relative', 'absolute'
+
     """
     def __init__(self, data, width=None, height=None,
                  left="0%", top="0%", position='relative'):
@@ -213,11 +221,11 @@ class Vega(Element):
         """Renders the HTML representation of the element."""
         self.json = json.dumps(self.data)
 
-        self._parent.html.add_children(Element(Template("""
+        self._parent.html.add_child(Element(Template("""
             <div id="{{this.get_name()}}"></div>
             """).render(this=self, kwargs=kwargs)), name=self.get_name())
 
-        self._parent.script.add_children(Element(Template("""
+        self._parent.script.add_child(Element(Template("""
             vega_parse({{this.json}},{{this.get_name()}});
             """).render(this=self)), name=self.get_name())
 
@@ -225,7 +233,7 @@ class Vega(Element):
         assert isinstance(figure, Figure), ("You cannot render this Element "
                                             "if it's not in a Figure.")
 
-        figure.header.add_children(Element(Template("""
+        figure.header.add_child(Element(Template("""
             <style> #{{this.get_name()}} {
                 position : {{this.position}};
                 width : {{this.width[0]}}{{this.width[1]}};
@@ -235,26 +243,27 @@ class Vega(Element):
             </style>
             """).render(this=self, **kwargs)), name=self.get_name())
 
-        figure.header.add_children(
+        figure.header.add_child(
             JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/d3/3.5.5/d3.min.js"),  # noqa
             name='d3')
 
-        figure.header.add_children(
+        figure.header.add_child(
             JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/vega/1.4.3/vega.min.js"),  # noqa
             name='vega')
 
-        figure.header.add_children(
+        figure.header.add_child(
             JavascriptLink("https://code.jquery.com/jquery-2.1.0.min.js"),
             name='jquery')
 
-        figure.script.add_children(
+        figure.script.add_child(
             Template("""function vega_parse(spec, div) {
             vg.parse.spec(spec, function(chart) { chart({el:div}).update(); });}"""),  # noqa
             name='vega_parse')
 
 
 class GeoJson(Layer):
-    """Creates a GeoJson object for plotting into a Map.
+    """
+    Creates a GeoJson object for plotting into a Map.
 
     Parameters
     ----------
@@ -273,12 +282,16 @@ class GeoJson(Layer):
         Adds the layer as an optional overlay (True) or the base layer (False).
     control : bool, default True
         Whether the Layer will be included in LayerControls
+    smooth_factor: float, default None
+        How much to simplify the polyline on each zoom level. More means
+        better performance and smoother look, and less means more accurate
+        representation. Leaflet defaults to 1.0.
 
     Examples
     --------
-    >>> # Providing file that shall be embeded.
+    >>> # Providing file that shall be embedded.
     >>> GeoJson(open('foo.json'))
-    >>> # Providing filename that shall not be embeded.
+    >>> # Providing filename that shall not be embedded.
     >>> GeoJson('foo.json')
     >>> # Providing dict.
     >>> GeoJson(json.load(open('foo.json')))
@@ -290,9 +303,11 @@ class GeoJson(Layer):
     ...                             x['properties']['name']=='Alabama' else
     ...                             '#00ff00'}
     >>> GeoJson(geojson, style_function=style_function)
+
     """
     def __init__(self, data, style_function=None, name=None,
-                 overlay=True, control=True):
+                 overlay=True, control=True, smooth_factor=None,
+                 highlight_function=None):
         super(GeoJson, self).__init__(name=name, overlay=overlay,
                                       control=control)
         self._name = 'GeoJson'
@@ -326,14 +341,54 @@ class GeoJson(Layer):
         if style_function is None:
             def style_function(x):
                 return {}
+
         self.style_function = style_function
 
+        self.highlight = highlight_function is not None
+
+        if highlight_function is None:
+            def highlight_function(x):
+                return {}
+
+        self.highlight_function = highlight_function
+
+        self.smooth_factor = smooth_factor
+
         self._template = Template(u"""
             {% macro script(this, kwargs) %}
+
+            {% if this.highlight %}
+                {{this.get_name()}}_onEachFeature = function onEachFeature(feature, layer) {
+                    layer.on({
+                        mouseout: function(e) {
+                            e.target.setStyle(e.target.feature.properties.style);},
+                        mouseover: function(e) {
+                            e.target.setStyle(e.target.feature.properties.highlight);},
+                        click: function(e) {
+                            {{this._parent.get_name()}}.fitBounds(e.target.getBounds());}
+                        });
+                };
+            {% endif %}
+
                 var {{this.get_name()}} = L.geoJson(
-                    {% if this.embed %}{{this.style_data()}}{% else %}"{{this.data}}"{% endif %})
-                    .addTo({{this._parent.get_name()}});
+                    {% if this.embed %}{{this.style_data()}}{% else %}"{{this.data}}"{% endif %}
+                    {% if this.smooth_factor is not none or this.highlight %}
+                        , {
+                        {% if this.smooth_factor is not none  %}
+                            smoothFactor:{{this.smooth_factor}}
+                        {% endif %}
+
+                        {% if this.highlight %}
+                            {% if this.smooth_factor is not none  %}
+                            ,
+                            {% endif %}
+                            onEachFeature: {{this.get_name()}}_onEachFeature
+                        {% endif %}
+                        }
+                    {% endif %}
+                    ).addTo({{this._parent.get_name()}});
                 {{this.get_name()}}.setStyle(function(feature) {return feature.properties.style;});
+
             {% endmacro %}
             """)  # noqa
 
@@ -352,11 +407,14 @@ class GeoJson(Layer):
 
         for feature in self.data['features']:
             feature.setdefault('properties', {}).setdefault('style', {}).update(self.style_function(feature))  # noqa
+            feature.setdefault('properties', {}).setdefault('highlight', {}).update(self.highlight_function(feature))  # noqa
         return json.dumps(self.data, sort_keys=True)
 
     def _get_self_bounds(self):
-        """Computes the bounds of the object itself (not including it's children)
+        """
+        Computes the bounds of the object itself (not including it's children)
         in the form [[lat_min, lon_min], [lat_max, lon_max]]
+
         """
         if not self.embed:
             raise ValueError('Cannot compute bounds of non-embedded GeoJSON.')
@@ -385,7 +443,8 @@ class GeoJson(Layer):
 
 
 class TopoJson(Layer):
-    """Creates a TopoJson object for plotting into a Map.
+    """
+    Creates a TopoJson object for plotting into a Map.
 
     Parameters
     ----------
@@ -407,6 +466,10 @@ class TopoJson(Layer):
         Adds the layer as an optional overlay (True) or the base layer (False).
     control : bool, default True
         Whether the Layer will be included in LayerControls
+    smooth_factor: float, default None
+        How much to simplify the polyline on each zoom level. More means
+        better performance and smoother look, and less means more accurate
+        representation. Leaflet defaults to 1.0.
 
     Examples
     --------
@@ -424,9 +487,10 @@ class TopoJson(Layer):
     ...                             x['properties']['name']=='Alabama' else
     ...                             '#00ff00'}
     >>> TopoJson(topo_json, 'object.myobject', style_function=style_function)
+
     """
     def __init__(self, data, object_path, style_function=None,
-                 name=None, overlay=True, control=True):
+                 name=None, overlay=True, control=True, smooth_factor=None):
         super(TopoJson, self).__init__(name=name, overlay=overlay,
                                        control=control)
         self._name = 'TopoJson'
@@ -447,13 +511,17 @@ class TopoJson(Layer):
                 return {}
         self.style_function = style_function
 
+        self.smooth_factor = smooth_factor
+
         self._template = Template(u"""
             {% macro script(this, kwargs) %}
                 var {{this.get_name()}}_data = {{this.style_data()}};
                 var {{this.get_name()}} = L.geoJson(topojson.feature(
                     {{this.get_name()}}_data,
-                    {{this.get_name()}}_data.{{this.object_path}}
-                    )).addTo({{this._parent.get_name()}});
+                    {{this.get_name()}}_data.{{this.object_path}})
+                        {% if this.smooth_factor is not none %}
+                            , {smoothFactor: {{this.smooth_factor}}}
+                        {% endif %}).addTo({{this._parent.get_name()}});
                 {{this.get_name()}}.setStyle(function(feature) {return feature.properties.style;});
 
             {% endmacro %}
@@ -483,13 +551,15 @@ class TopoJson(Layer):
         assert isinstance(figure, Figure), ("You cannot render this Element "
                                             "if it's not in a Figure.")
 
-        figure.header.add_children(
+        figure.header.add_child(
             JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/topojson/1.6.9/topojson.min.js"),  # noqa
             name='topojson')
 
     def _get_self_bounds(self):
-        """Computes the bounds of the object itself (not including it's children)
+        """
+        Computes the bounds of the object itself (not including it's children)
         in the form [[lat_min, lon_min], [lat_max, lon_max]]
+
         """
         if not self.embed:
             raise ValueError('Cannot compute bounds of non-embedded TopoJSON.')
@@ -519,8 +589,9 @@ class TopoJson(Layer):
 
 
 class MarkerCluster(Layer):
-    """Creates a MarkerCluster element to append into a map with
-    Map.add_children.
+    """
+    Creates a MarkerCluster element to append into a map with
+    Map.add_child.
 
     Parameters
     ----------
@@ -530,6 +601,7 @@ class MarkerCluster(Layer):
         Adds the layer as an optional overlay (True) or the base layer (False).
     control : bool, default True
         Whether the Layer will be included in LayerControls
+
     """
     def __init__(self, name=None, overlay=True, control=True):
         super(MarkerCluster, self).__init__(name=name, overlay=overlay,
@@ -549,20 +621,20 @@ class MarkerCluster(Layer):
         figure = self.get_root()
         assert isinstance(figure, Figure), ("You cannot render this Element "
                                             "if it's not in a Figure.")
-        figure.header.add_children(
-            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/0.4.0/leaflet.markercluster-src.js"),  # noqa
+        figure.header.add_child(
+            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/leaflet.markercluster-src.js"),  # noqa
             name='marker_cluster_src')
 
-        figure.header.add_children(
-            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/0.4.0/leaflet.markercluster.js"),  # noqa
+        figure.header.add_child(
+            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/leaflet.markercluster.js"),  # noqa
             name='marker_cluster')
 
-        figure.header.add_children(
-            CssLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/0.4.0/MarkerCluster.css"),  # noqa
+        figure.header.add_child(
+            CssLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/MarkerCluster.css"),  # noqa
             name='marker_cluster_css')
 
-        figure.header.add_children(
-            CssLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/0.4.0/MarkerCluster.Default.css"),  # noqa
+        figure.header.add_child(
+            CssLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/MarkerCluster.Default.css"),  # noqa
             name="marker_cluster_default_css")
 
 
@@ -592,6 +664,7 @@ class DivIcon(MacroElement):
 
     For more information see:
     http://leafletjs.com/reference.html#divicon
+
     """
     def __init__(self, html=None, icon_size=None, icon_anchor=None,
                  popup_anchor=None, class_name='empty'):
@@ -618,28 +691,31 @@ class DivIcon(MacroElement):
             """)  # noqa
 
 
-class CircleMarker(Marker):
-    """Creates a CircleMarker object for plotting on a Map.
+class Circle(Marker):
+    """
+    Creates a Circle object for plotting on a Map.
 
     Parameters
     ----------
     location: tuple or list, default None
         Latitude and Longitude of Marker (Northing, Easting)
     radius: int
-        The radius of the circle in pixels.
+        The radius of the circle in meters. For setting the radius in pixel,
+        use CircleMarker.
     color: str, default 'black'
         The color of the marker's edge in a HTML-compatible format.
     fill_color: str, default 'black'
         The fill color of the marker in a HTML-compatible format.
-    fill_opacity: float, default à.6
+    fill_opacity: float, default 0.6
         The fill opacity of the marker, between 0. and 1.
     popup: string or folium.Popup, default None
         Input text or visualization for object.
+
     """
     def __init__(self, location, radius=500, color='black',
                  fill_color='black', fill_opacity=0.6, popup=None):
-        super(CircleMarker, self).__init__(location, popup=popup)
-        self._name = 'CircleMarker'
+        super(Circle, self).__init__(location, popup=popup)
+        self._name = 'Circle'
         self.radius = radius
         self.color = color
         self.fill_color = fill_color
@@ -662,6 +738,171 @@ class CircleMarker(Marker):
             """)
 
 
+class CircleMarker(Marker):
+    """
+    Creates a CircleMarker object for plotting on a Map.
+
+    Parameters
+    ----------
+    location: tuple or list, default None
+        Latitude and Longitude of Marker (Northing, Easting)
+    radius: int
+        The radius of the circle in pixels. For setting the radius in meter,
+        use Circle.
+    color: str, default 'black'
+        The color of the marker's edge in a HTML-compatible format.
+    weight: int, default 2
+        Stroke weight in pixels
+    fill_color: str, default 'black'
+        The fill color of the marker in a HTML-compatible format.
+    fill_opacity: float, default 0.6
+        The fill opacity of the marker, between 0. and 1.
+    popup: string or folium.Popup, default None
+        Input text or visualization for object.
+
+    """
+    def __init__(self, location, radius=500, color='black',
+                 weight=2, fill_color='black', fill_opacity=0.6,
+                 popup=None):
+        super(CircleMarker, self).__init__(location, popup=popup)
+        self._name = 'CircleMarker'
+        self.radius = radius
+        self.color = color
+        self.weight = weight
+        self.fill_color = fill_color
+        self.fill_opacity = fill_opacity
+
+        self._template = Template(u"""
+            {% macro script(this, kwargs) %}
+
+            var {{this.get_name()}} = L.circleMarker(
+                [{{this.location[0]}},{{this.location[1]}}],
+                {
+                    color: '{{ this.color }}',
+                    weight: {{ this.weight }},
+                    fillColor: '{{ this.fill_color }}',
+                    fillOpacity: {{ this.fill_opacity }}
+                    }
+                )
+                .setRadius({{ this.radius }})
+                .addTo({{this._parent.get_name()}});
+            {% endmacro %}
+            """)
+
+
+class RectangleMarker(Marker):
+    def __init__(self, bounds, color='black', weight=1, fill_color='black',
+                 fill_opacity=0.6, popup=None):
+        """
+        Creates a RectangleMarker object for plotting on a Map.
+
+        Parameters
+        ----------
+        bounds: tuple or list, default None
+            Latitude and Longitude of Marker (southWest and northEast)
+        color: string, default ('black')
+            Edge color of a rectangle.
+        weight: float, default (1)
+            Edge line width of a rectangle.
+        fill_color: string, default ('black')
+            Fill color of a rectangle.
+        fill_opacity: float, default (0.6)
+            Fill opacity of a rectangle.
+        popup: string or folium.Popup, default None
+            Input text or visualization for object.
+
+        Returns
+        -------
+        folium.features.RectangleMarker object
+
+        Example
+        -------
+        >>> RectangleMarker(
+        ...  bounds=[[35.681, 139.766], [35.691, 139.776]],
+        ...  color='blue', fill_color='red', popup='Tokyo, Japan'
+        ... )
+
+        """
+        super(RectangleMarker, self).__init__(bounds, popup=popup)
+        self._name = 'RectangleMarker'
+        self.color = color
+        self.weight = weight
+        self.fill_color = fill_color
+        self.fill_opacity = fill_opacity
+        self._template = Template(u"""
+            {% macro script(this, kwargs) %}
+            var {{this.get_name()}} = L.rectangle(
+                                  [[{{this.location[0]}},{{this.location[1]}}],
+                                  [{{this.location[2]}},{{this.location[3]}}]],
+                {
+                    color: '{{ this.color }}',
+                    fillColor: '{{ this.fill_color }}',
+                    fillOpacity: {{ this.fill_opacity }},
+                    weight: {{ this.weight }}
+                }).addTo({{this._parent.get_name()}});
+
+            {% endmacro %}
+            """)
+
+
+class PolygonMarker(Marker):
+    def __init__(self, locations, color='black', weight=1, fill_color='black',
+                 fill_opacity=0.6, popup=None, latlon=True):
+        """
+        Creates a PolygonMarker object for plotting on a Map.
+
+        Parameters
+        ----------
+        locations: tuple or list, default None
+            Latitude and Longitude of Polygon
+        color: string, default ('black')
+            Edge color of a polygon.
+        weight: float, default (1)
+            Edge line width of a polygon.
+        fill_color: string, default ('black')
+            Fill color of a polygon.
+        fill_opacity: float, default (0.6)
+            Fill opacity of a polygon.
+        popup: string or folium.Popup, default None
+            Input text or visualization for object.
+
+        Returns
+        -------
+        folium.features.Polygon object
+
+        Examples
+        --------
+        >>> locations = [[35.6762, 139.7795],
+        ...              [35.6718, 139.7831],
+        ...              [35.6767, 139.7868],
+        ...              [35.6795, 139.7824],
+        ...              [35.6787, 139.7791]]
+        >>> Polygon(locations, color='blue', weight=10, fill_color='red',
+        ...         fill_opacity=0.5, popup='Tokyo, Japan'))
+
+        """
+        super(PolygonMarker, self).__init__((
+            _locations_mirror(locations) if not latlon else
+            _locations_tolist(locations)), popup=popup
+        )
+        self._name = 'PolygonMarker'
+        self.color = color
+        self.weight = weight
+        self.fill_color = fill_color
+        self.fill_opacity = fill_opacity
+        self._template = Template(u"""
+            {% macro script(this, kwargs) %}
+            var {{this.get_name()}} = L.polygon({{this.location}},
+                {
+                    color: '{{ this.color }}',
+                    fillColor: '{{ this.fill_color }}',
+                    fillOpacity: {{ this.fill_opacity }},
+                    weight: {{ this.weight }}
+                }).addTo({{this._parent.get_name()}});
+            {% endmacro %}
+            """)
+
+
 class LatLngPopup(MacroElement):
     """
     When one clicks on a Map that contains a LatLngPopup,
@@ -688,14 +929,16 @@ class LatLngPopup(MacroElement):
 
 
 class ClickForMarker(MacroElement):
-    """When one clicks on a Map that contains a ClickForMarker, a Marker is created
-    at the pointer's position.
+    """
+    When one clicks on a Map that contains a ClickForMarker,
+    a Marker is created at the pointer's position.
 
     Parameters
     ----------
     popup: str, default None
         Text to display in the markers' popups.
         If None, the popups will display the marker's latitude and longitude.
+
     """
     def __init__(self, popup=None):
         super(ClickForMarker, self).__init__()
@@ -724,7 +967,7 @@ class ClickForMarker(MacroElement):
 class PolyLine(MacroElement):
     """
     Creates a PolyLine object to append into a map with
-    Map.add_children.
+    Map.add_child.
 
     Parameters
     ----------
@@ -740,6 +983,7 @@ class PolyLine(MacroElement):
         while Leaflet polyline's default is latlon=True.
     popup: string or folium.Popup, default None
         Input text or visualization for object.
+
     """
     def __init__(self, locations, color=None, weight=None,
                  opacity=None, latlon=True, popup=None):
@@ -751,9 +995,9 @@ class PolyLine(MacroElement):
         self.weight = weight
         self.opacity = opacity
         if isinstance(popup, text_type) or isinstance(popup, binary_type):
-            self.add_children(Popup(popup))
+            self.add_child(Popup(popup))
         elif popup is not None:
-            self.add_children(popup)
+            self.add_child(popup)
 
         self._template = Template(u"""
             {% macro script(this, kwargs) %}
@@ -769,74 +1013,10 @@ class PolyLine(MacroElement):
             """)  # noqa
 
     def _get_self_bounds(self):
-        """Computes the bounds of the object itself (not including it's children)
-        in the form [[lat_min, lon_min], [lat_max, lon_max]]
         """
-        bounds = [[None, None], [None, None]]
-        for point in iter_points(self.data):
-            bounds = [
-                [
-                    none_min(bounds[0][0], point[0]),
-                    none_min(bounds[0][1], point[1]),
-                ],
-                [
-                    none_max(bounds[1][0], point[0]),
-                    none_max(bounds[1][1], point[1]),
-                ],
-            ]
-        return bounds
-
-
-class MultiPolyLine(MacroElement):
-    """
-    Creates a MultiPolyLine object to append into a map with
-    Map.add_children.
-
-    Parameters
-    ----------
-    locations: list of points (latitude, longitude)
-        Latitude and Longitude of line (Northing, Easting)
-    color: string, default Leaflet's default ('#03f')
-    weight: float, default Leaflet's default (5)
-    opacity: float, default Leaflet's default (0.5)
-    latlon: bool, default True
-        Whether locations are given in the form [[lat, lon]]
-        or not ([[lon, lat]] if False).
-        Note that the default GeoJson format is latlon=False,
-        while Leaflet polyline's default is latlon=True.
-    popup: string or folium.Popup, default None
-        Input text or visualization for object.
-    """
-    def __init__(self, locations, color=None, weight=None,
-                 opacity=None, latlon=True, popup=None):
-        super(MultiPolyLine, self).__init__()
-        self._name = 'MultiPolyLine'
-        self.data = (_locations_mirror(locations) if not latlon else
-                     _locations_tolist(locations))
-        self.color = color
-        self.weight = weight
-        self.opacity = opacity
-        if isinstance(popup, text_type) or isinstance(popup, binary_type):
-            self.add_children(Popup(popup))
-        elif popup is not None:
-            self.add_children(popup)
-
-        self._template = Template(u"""
-            {% macro script(this, kwargs) %}
-                var {{this.get_name()}} = L.multiPolyline(
-                    {{this.data}},
-                    {
-                        {% if this.color != None %}color: '{{ this.color }}',{% endif %}
-                        {% if this.weight != None %}weight: {{ this.weight }},{% endif %}
-                        {% if this.opacity != None %}opacity: {{ this.opacity }},{% endif %}
-                        });
-                {{this._parent.get_name()}}.addLayer({{this.get_name()}});
-            {% endmacro %}
-            """)  # noqa
-
-    def _get_self_bounds(self):
-        """Computes the bounds of the object itself (not including it's children)
+        Computes the bounds of the object itself (not including it's children)
         in the form [[lat_min, lon_min], [lat_max, lon_max]]
+
         """
         bounds = [[None, None], [None, None]]
         for point in iter_points(self.data):
@@ -854,7 +1034,8 @@ class MultiPolyLine(MacroElement):
 
 
 class CustomIcon(Icon):
-    """Create a custom icon, based on an image.
+    """
+    Create a custom icon, based on an image.
 
     Parameters
     ----------
@@ -883,6 +1064,7 @@ class CustomIcon(Icon):
     popup_anchor : tuple of 2 int
         The coordinates of the point from which popups will "open",
         relative to the icon anchor.
+
     """
     def __init__(self, icon_image, icon_size=None, icon_anchor=None,
                  shadow_image=None, shadow_size=None, shadow_anchor=None,
@@ -916,3 +1098,58 @@ class CustomIcon(Icon):
                 {{this._parent.get_name()}}.setIcon({{this.get_name()}});
             {% endmacro %}
             """)  # noqa
+
+
+class ColorLine(FeatureGroup):
+    """
+    Draw data on a map with specified colors.
+
+    Parameters
+    ----------
+    positions: tuple or list
+        The list of points latitude and longitude
+    colors: tuple or list
+        The list of segments colors.
+        It must have length equal to `len(positions)-1`.
+    colormap: branca.colormap.Colormap or list or tuple
+        The colormap to use. If a list or tuple of colors is provided,
+        a LinearColormap will be created from it.
+    nb_steps: int, default 12
+        To have lighter output the colormap will be discretized
+        to that number of colors.
+    opacity: float, default 1
+        Line opacity, scale 0-1
+    weight: int, default 2
+        Stroke weight in pixels
+    **kwargs
+        Further parameters available. See folium.map.FeatureGroup
+
+    Returns
+    -------
+    A ColorLine object that you can `add_to` a Map.
+
+    """
+    def __init__(self, positions, colors, colormap=None, nb_steps=12,
+                 weight=None, opacity=None, **kwargs):
+        super(ColorLine, self).__init__(**kwargs)
+        self._name = 'ColorLine'
+
+        if colormap is None:
+            cm = LinearColormap(['green', 'yellow', 'red'],
+                                vmin=min(colors),
+                                vmax=max(colors),
+                                ).to_step(nb_steps)
+        elif isinstance(colormap, LinearColormap):
+            cm = colormap.to_step(nb_steps)
+        elif isinstance(colormap, list) or isinstance(colormap, tuple):
+            cm = LinearColormap(colormap,
+                                vmin=min(colors),
+                                vmax=max(colors),
+                                ).to_step(nb_steps)
+        else:
+            cm = colormap
+        out = {}
+        for (lat1, lng1), (lat2, lng2), color in zip(positions[:-1], positions[1:], colors):  # noqa
+            out.setdefault(cm(color), []).append([[lat1, lng1], [lat2, lng2]])
+        for key, val in out.items():
+            self.add_child(PolyLine(val, color=key, weight=weight, opacity=opacity))  # noqa
