@@ -9,9 +9,10 @@ Create a HeatMap layer
 import json
 from jinja2 import Template
 
-from folium.element import JavascriptLink, Figure
+from branca.element import JavascriptLink, Figure
+from branca.utilities import none_min, none_max
+
 from folium.map import TileLayer
-from folium.utilities import none_min, none_max
 
 
 class HeatMap(TileLayer):
@@ -77,7 +78,7 @@ class HeatMap(TileLayer):
         assert isinstance(figure, Figure), ("You cannot render this Element "
                                             "if it's not in a Figure.")
 
-        figure.header.add_children(
+        figure.header.add_child(
             JavascriptLink("https://leaflet.github.io/Leaflet.heat/dist/leaflet-heat.js"),  # noqa
             name='leaflet-heat.js')
 
