@@ -9,7 +9,7 @@ and night regions on maps.
 """
 from jinja2 import Template
 
-from folium.element import JavascriptLink, MacroElement, Figure
+from branca.element import JavascriptLink, MacroElement, Figure
 
 
 class Terminator(MacroElement):
@@ -42,6 +42,6 @@ class Terminator(MacroElement):
         assert isinstance(figure, Figure), ("You cannot render this Element "
                                             "if it's not in a Figure.")
 
-        figure.header.add_children(
+        figure.header.add_child(
             JavascriptLink("https://rawgithub.com/joergdietrich/Leaflet.Terminator/master/L.Terminator.js"),  # noqa
             name='markerclusterjs')
